@@ -5,7 +5,6 @@ import archinstall
 from archinstall import Installer
 from archinstall import profile
 from archinstall import SysInfo
-from archinstall import mirrors
 from archinstall import disk
 from archinstall import menu
 from archinstall import models
@@ -109,12 +108,8 @@ def perform_installation(mountpoint: Path):
 				# generate encryption key files for the mounted luks devices
 				installation.generate_key_files()
 
-		# Set mirrors used by pacstrap (outside of installation)
 		if mirror_config := archinstall.arguments.get('mirror_config', None):
-			if mirror_config.mirror_regions:
-				mirrors.use_mirrors(mirror_config.mirror_regions)
-			if mirror_config.custom_mirrors:
-				mirrors.add_custom_mirrors(mirror_config.custom_mirrors)
+			installation.set_mirrors(mirror_config)
 
 		installation.minimal_installation(
 			testing=enable_testing,
@@ -124,7 +119,7 @@ def perform_installation(mountpoint: Path):
 		)
 
 		if mirror_config := archinstall.arguments.get('mirror_config', None):
-			installation.set_mirrors(mirror_config)  # Set the mirrors in the installation medium
+			installation.set_mirrors(mirror_config, on_target=True)
 
 		if archinstall.arguments.get('swap'):
 			installation.setup_swap('zram')
@@ -171,10 +166,6 @@ def perform_installation(mountpoint: Path):
 		if (root_pw := archinstall.arguments.get('!root-password', None)) and len(root_pw):
 			installation.user_set_pw('root', root_pw)
 
-		# This step must be after profile installs to allow profiles_bck to install language pre-requisites.
-		# After which, this step will set the language both for console and x11 if x11 was installed for instance.
-		installation.set_keyboard_language(locale_config.kb_layout)
-
 		if profile_config := archinstall.arguments.get('profile_config', None):
 			profile_config.profile.post_install(installation)
 
