@@ -15,7 +15,7 @@ from .hardware import SysInfo
 from .locale import LocaleConfiguration
 from .locale import verify_keyboard_layout, verify_x11_keyboard_layout
 from .luks import Luks2
-from .mirrors import use_mirrors, MirrorConfiguration, add_custom_mirrors
+from .mirrors import MirrorConfiguration
 from .models.bootloader import Bootloader
 from .models.network_configuration import Nic
 from .models.users import User
@@ -83,7 +83,7 @@ class Installer:
 		# systemd, sd-vconsole and sd-encrypt will be replaced by udev, keymap and encrypt
 		# if HSM is not used to encrypt the root volume. Check mkinitcpio() function for that override.
 		self._hooks: List[str] = [
-			"base", "systemd", "autodetect", "keyboard",
+			"base", "systemd", "autodetect", "microcode", "keyboard",
 			"sd-vconsole", "modconf", "block", "filesystems", "fsck"
 		]
 		self._kernel_params: List[str] = []
@@ -148,7 +148,7 @@ class Installer:
 				if not _notified and time.time() - _started_wait > 5:
 					_notified = True
 					warn(
-						_("Time syncronization not completing, while you wait - check the docs for workarounds: https://archinstall.readthedocs.io/"))
+						_("Time synchronization not completing, while you wait - check the docs for workarounds: https://archinstall.readthedocs.io/"))
 
 				time_val = SysCommand('timedatectl show --property=NTPSynchronized --value').decode()
 				if time_val and time_val.strip() == 'yes':
@@ -240,7 +240,11 @@ class Installer:
 			disk.device_handler.mount(part_mod.dev_path, target, options=part_mod.mount_options)
 
 		if part_mod.fs_type == disk.FilesystemType.Btrfs and part_mod.dev_path:
-			self._mount_btrfs_subvol(part_mod.dev_path, part_mod.btrfs_subvols)
+			self._mount_btrfs_subvol(
+				part_mod.dev_path,
+				part_mod.btrfs_subvols,
+				part_mod.mount_options
+			)
 
 	def _mount_luks_partition(self, part_mod: disk.PartitionModification, luks_handler: Luks2):
 		# it would be none if it's btrfs as the subvolumes will have the mountpoints defined
@@ -251,11 +255,18 @@ class Installer:
 		if part_mod.fs_type == disk.FilesystemType.Btrfs and luks_handler.mapper_dev:
 			self._mount_btrfs_subvol(luks_handler.mapper_dev, part_mod.btrfs_subvols)
 
-	def _mount_btrfs_subvol(self, dev_path: Path, subvolumes: List[disk.SubvolumeModification]):
+	def _mount_btrfs_subvol(
+		self,
+		dev_path: Path,
+		subvolumes: List[disk.SubvolumeModification],
+		mount_options: List[str] = []
+	):
 		for subvol in subvolumes:
-			mountpoint = self.target / subvol.relative_mountpoint
-			mount_options = subvol.mount_options + [f'subvol={subvol.name}']
-			disk.device_handler.mount(dev_path, mountpoint, options=mount_options)
+			disk.device_handler.mount(
+				dev_path,
+				self.target / subvol.relative_mountpoint,
+				options=mount_options + [f'subvol={subvol.name}']
+			)
 
 	def generate_key_files(self):
 		for part_mod in self._disk_encryption.partitions:
@@ -318,29 +329,56 @@ class Installer:
 	def post_install_check(self, *args: str, **kwargs: str) -> List[str]:
 		return [step for step, flag in self.helper_flags.items() if flag is False]
 
-	def set_mirrors(self, mirror_config: MirrorConfiguration):
+	def set_mirrors(self, mirror_config: MirrorConfiguration, on_target: bool = False):
+		"""
+		Set the mirror configuration for the installation.
+
+		:param mirror_config: The mirror configuration to use.
+		:type mirror_config: MirrorConfiguration
+
+		:on_target: Whether to set the mirrors on the target system or the live system.
+		:param on_target: bool
+		"""
+		debug('Setting mirrors')
+
 		for plugin in plugins.values():
 			if hasattr(plugin, 'on_mirrors'):
 				if result := plugin.on_mirrors(mirror_config):
 					mirror_config = result
 
-		destination = f'{self.target}/etc/pacman.d/mirrorlist'
-		if mirror_config.mirror_regions:
-			use_mirrors(mirror_config.mirror_regions, destination)
-		if mirror_config.custom_mirrors:
-			add_custom_mirrors(mirror_config.custom_mirrors)
+		if on_target:
+			local_pacman_conf = Path(f'{self.target}/etc/pacman.conf')
+			local_mirrorlist_conf = Path(f'{self.target}/etc/pacman.d/mirrorlist')
+		else:
+			local_pacman_conf = Path('/etc/pacman.conf')
+			local_mirrorlist_conf = Path('/etc/pacman.d/mirrorlist')
+
+		mirrorlist_config = mirror_config.mirrorlist_config()
+		pacman_config = mirror_config.pacman_config()
+
+		if pacman_config:
+			debug(f'Pacman config: {pacman_config}')
+
+			with local_pacman_conf.open('a') as fp:
+				fp.write(pacman_config)
+
+		if mirrorlist_config:
+			debug(f'Mirrorlist: {mirrorlist_config}')
+
+			with local_mirrorlist_conf.open('a') as fp:
+				fp.write(mirrorlist_config)
 
 	def genfstab(self, flags: str = '-pU'):
 		fstab_path = self.target / "etc" / "fstab"
 		info(f"Updating {fstab_path}")
 
 		try:
-			gen_fstab = SysCommand(f'/usr/bin/genfstab {flags} {self.target}').decode()
+			gen_fstab = SysCommand(f'/usr/bin/genfstab {flags} {self.target}').output()
 		except SysCallError as err:
 			raise RequirementError(
 				f'Could not generate fstab, strapping in packages most likely failed (disk out of space?)\n Error: {err}')
 
-		with open(fstab_path, 'a') as fp:
+		with open(fstab_path, 'ab') as fp:
 			fp.write(gen_fstab)
 
 		if not fstab_path.is_file():
@@ -355,37 +393,6 @@ class Installer:
 			for entry in self._fstab_entries:
 				fp.write(f'{entry}\n')
 
-		for mod in self._disk_config.device_modifications:
-			for part_mod in mod.partitions:
-				if part_mod.fs_type != disk.FilesystemType.Btrfs:
-					continue
-
-				with fstab_path.open('r') as fp:
-					fstab = fp.readlines()
-
-				# Replace the {installation}/etc/fstab with entries
-				# using the compress=zstd where the mountpoint has compression set.
-				for index, line in enumerate(fstab):
-					# So first we grab the mount options by using subvol=.*? as a locator.
-					# And we also grab the mountpoint for the entry, for instance /var/log
-					subvoldef = re.findall(',.*?subvol=.*?[\t ]', line)
-					mountpoint = re.findall('[\t ]/.*?[\t ]', line)
-
-					if not subvoldef or not mountpoint:
-						continue
-
-					for sub_vol in part_mod.btrfs_subvols:
-						# We then locate the correct subvolume and check if it's compressed,
-						# and skip entries where compression is already defined
-						# We then sneak in the compress=zstd option if it doesn't already exist:
-						if sub_vol.compress and str(sub_vol.mountpoint) == Path(
-							mountpoint[0].strip()) and ',compress=zstd,' not in line:
-							fstab[index] = line.replace(subvoldef[0], f',compress=zstd{subvoldef[0]}')
-							break
-
-				with fstab_path.open('w') as fp:
-					fp.writelines(fstab)
-
 	def set_hostname(self, hostname: str, *args: str, **kwargs: str) -> None:
 		with open(f'{self.target}/etc/hostname', 'w') as fh:
 			fh.write(hostname + '\n')
@@ -661,8 +668,6 @@ class Installer:
 		if testing:
 			info("The testing flag is set. This system will be installed with testing repositories enabled.")
 			pacman_conf.enable(pacman.Repo.Testing)
-			if multilib:
-				pacman_conf.enable(pacman.Repo.MultilibTesting)
 		else:
 			info("The testing flag is not set. This system will be installed without testing repositories enabled.")
 
@@ -689,6 +694,7 @@ class Installer:
 		# sys_command('/usr/bin/arch-chroot /mnt hwclock --hctosys --localtime')
 		self.set_hostname(hostname)
 		self.set_locale(locale_config)
+		self.set_keyboard_language(locale_config.kb_layout)
 
 		# TODO: Use python functions for this
 		SysCommand(f'/usr/bin/arch-chroot {self.target} chmod 700 /root')
@@ -876,13 +882,6 @@ class Installer:
 			f'# Created on: {self.init_time}'
 		)
 
-		microcode = []
-
-		if ucode := self._get_microcode():
-			microcode.append(f'initrd  /{ucode}')
-		else:
-			debug('Archinstall will not add any ucode to systemd-boot config.')
-
 		options = 'options ' + ' '.join(self._get_kernel_params(root_partition))
 
 		for kernel in self.kernels:
@@ -892,7 +891,6 @@ class Installer:
 					*comments,
 					f'title   Arch Linux ({kernel}{variant})',
 					f'linux   /vmlinuz-{kernel}',
-					*microcode,
 					f'initrd  /initramfs-{kernel}{variant}.img',
 					options,
 				]
@@ -921,12 +919,7 @@ class Installer:
 
 		info(f"GRUB boot partition: {boot_partition.dev_path}")
 
-		if boot_partition == root_partition and root_partition.mountpoint:
-			boot_dir = root_partition.mountpoint / 'boot'
-		elif boot_partition.mountpoint:
-			boot_dir = boot_partition.mountpoint
-		else:
-			raise ValueError('Could not detect boot directory')
+		boot_dir = Path('/boot')
 
 		command = [
 			'/usr/bin/arch-chroot',
@@ -944,8 +937,9 @@ class Installer:
 			self.pacman.strap('efibootmgr') # TODO: Do we need? Yes, but remove from minimal_installation() instead?
 
 			boot_dir_arg = []
-			if boot_partition != efi_partition:
-				boot_dir_arg.append(f'--boot-directory={boot_dir}')
+			if boot_partition.mountpoint and boot_partition.mountpoint != boot_dir:
+				boot_dir_arg.append(f'--boot-directory={boot_partition.mountpoint}')
+				boot_dir = boot_partition.mountpoint
 
 			add_options = [
 				'--target=x86_64-efi',
@@ -1058,11 +1052,6 @@ Exec = /bin/sh -c "{hook_command}"
 		hook_path = hooks_dir / '99-limine.hook'
 		hook_path.write_text(hook_contents)
 
-		microcode = []
-
-		if ucode := self._get_microcode():
-			microcode = [f'MODULE_PATH=boot:///{ucode}']
-
 		kernel_params = ' '.join(self._get_kernel_params(root_partition))
 		config_contents = 'TIMEOUT=5\n'
 
@@ -1071,7 +1060,6 @@ Exec = /bin/sh -c "{hook_command}"
 				entry = [
 					f'PROTOCOL=linux',
 					f'KERNEL_PATH=boot:///vmlinuz-{kernel}',
-					*microcode,
 					f'MODULE_PATH=boot:///initramfs-{kernel}{variant}.img',
 					f'CMDLINE={kernel_params}',
 				]
@@ -1102,15 +1090,7 @@ Exec = /bin/sh -c "{hook_command}"
 		if not uki_enabled:
 			loader = '/vmlinuz-{kernel}'
 
-			microcode = []
-
-			if ucode := self._get_microcode():
-				microcode.append(f'initrd=/{ucode}')
-			else:
-				debug('Archinstall will not add any ucode to firmware boot entry.')
-
 			entries = (
-				*microcode,
 				'initrd=/initramfs-{kernel}.img',
 				*self._get_kernel_params(root_partition)
 			)
@@ -1153,8 +1133,6 @@ Exec = /bin/sh -c "{hook_command}"
 			kernel_parameters = self._get_kernel_params(root_partition)
 			cmdline.write(' '.join(kernel_parameters) + '\n')
 
-		ucode = self._get_microcode()
-
 		diff_mountpoint = None
 
 		if efi_partition.mountpoint != Path('/efi'):
@@ -1169,10 +1147,8 @@ Exec = /bin/sh -c "{hook_command}"
 			config = preset.read_text().splitlines(True)
 
 			for index, line in enumerate(config):
-				if not ucode and line.startswith('ALL_microcode='):
-					config[index] = '#' + line
 				# Avoid storing redundant image file
-				elif m := image_re.match(line):
+				if m := image_re.match(line):
 					image = self.target / m.group(2)
 					image.unlink(missing_ok=True)
 					config[index] = '#' + m.group(1)
